/** \file QuasiCrystal.h */

#ifndef QUASI_CRYSTAL_H
#define QUASI_CRYSTAL_H

#include "AMDiS.h"
#include "BaseProblem.h"
#include "ExtendedProblemStat.h"

namespace AMDiS { namespace base_problems {

namespace detail {

  /** Quasicrystal Phase-field Crystal problem
  */
  template<typename ProblemStatType>
  class QuasiCrystal : public BaseProblem<ProblemStatType>
  {
  public: // typedefs

    typedef QuasiCrystal<ProblemStatType> self;
    typedef BaseProblem<ProblemStatType> super;

  public:

    QuasiCrystal(const std::string &name_);
    ~QuasiCrystal() {};

    virtual void fillOperators() override;
    virtual void fillBoundaryConditions() override {};
    
    virtual void finalizeData() override;

  protected:

    bool useMobility;

    double q;
    double r;
    double c;
    double alpha;
    double density;
    
    double two;
    double minus2;
    double three;
    double minus3;
  };

} // end namespace detail

typedef detail::QuasiCrystal<AMDiS::ProblemStat> QuasiCrystal;

} }

#include "QuasiCrystal.hh"


#endif // PHASE_FIELD_CRYSTAL_H
