#include "VacancyPhaseFieldCrystal_Base.h"

namespace AMDiS { namespace base_problems {
  
using namespace std;

VacancyPhaseFieldCrystal::VacancyPhaseFieldCrystal(const std::string &name_, bool createProblem) :
  super(name_, createProblem),
  r(-0.4),	// temperature deviation
  rho(-0.3),	// mean density
  two(2.0),
  three(3.0),
  minus2(-3.0),
  four(4.0),
  minu4(-4.0),
  eight(8.0)
{  
  Parameters::get(name + "->r",r);
  Parameters::get(name + "->mean value", rho);
};


void VacancyPhaseFieldCrystal::fillOperators()
{ FUNCNAME("VacancyPhaseFieldCrystal::fillOperators()");
  
  // phi*rho
  Operator *opM = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opM->addTerm(new Simple_ZOT);
  Operator *opMold = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMold->addTerm(new VecAtQP_ZOT(prob->getSolution()->getDOFVector(0), NULL));
  
  // 0 = nu-laplace(rho)
  // -------------------
  // -2*div(rho*grad(rho))
  Operator *opL = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opL->addTerm(new VecAtQP_SOT(prob->getSolution()->getDOFVector(0)));
  opL->setUhOld(prob->getSolution()->getDOFVector(0));  
  prob->addMatrixOperator(opL, 0, 0, &two, &two); // -div(rho^old*grad(rho))  
  prob->addVectorOperator(opL, 0, &two, &two); // div(rho^old*grad(rho^old))  
  
  Operator *opRhoGradRho1 = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opRhoGradRho1->addTerm(new VecGrad_FOT(prob->getSolution()->getDOFVector(0), 
					 prob->getSolution()->getDOFVector(0), 
					 new AMDiS::Mult2<WorldVector<double>, double, WorldVector<double> >), GRD_PSI);
  prob->addMatrixOperator(opRhoGradRho1, 0, 0, &two, &two); // -div(rho*grad(rho^old))  
  
  // nu
  prob->addMatrixOperator(opM, 0, 2);
  
  // dt(rho) = laplace(mu) - u*grad(rho)
  // -----------------------------------
  double M0 = 1.0;
  Parameters::get(name + "->M0", M0);
  Operator *opLM = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opLM->addTerm(new Simple_SOT(M0));
  
  prob->addMatrixOperator(opM, 1, 0, getInvTau(), getInvTau());
  prob->addVectorOperator(opMold, 1, getInvTau(), getInvTau());
  prob->addMatrixOperator(opLM, 1, 1); 					// -laplace(mu)
  
  // ----------------------------------------------------------------------
  
  // -(2+2r)*rho^3
  Operator *opMPow3Expl = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMPowExpl->addTerm(new VecAtQP_ZOT(prob->getSolution()->getDOFVector(0),new AMDiS::Pow<3>(-4.0*(1.0+r)))); // -4*(1+r)*rho_old^3
  prob->addMatrixOperator(opMPow3Impl, 2, 0); 
  Operator *opMPow3Impl = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMPowImpl->addTerm(new VecAtQP_ZOT(prob->getSolution()->getDOFVector(0),new AMDiS::Pow<2>(-6.0*(1.0+r)))); // -6*(1+r)*rho_old^2
  prob->addVectorOperator(opMPow3Expl, 2);
  
  // -2*rho^7
  Operator *opMPow7Expl = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMPowExpl->addTerm(new VecAtQP_ZOT(prob->getSolution()->getDOFVector(0),new AMDiS::Pow<7>(-12.0))); // -12*rho_old^7
  prob->addMatrixOperator(opMPow7Impl, 2, 0); 
  Operator *opMPow7Impl = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMPowImpl->addTerm(new VecAtQP_ZOT(prob->getSolution()->getDOFVector(0),new AMDiS::Pow<6>(-14.0))); // -14*rho_old^6
  prob->addVectorOperator(opMPow7Expl, 2);
  
  // -3*rho*nu
  Operator *opRhoNu = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opRhoNu->addTerm(new VecAtQP_ZOT(prob->getSolution()->getDOFVector(0), NULL));
  prob->addMatrixOperator(opRhoNu, 2, 2, &minus3, &minus3); 			
  Operator *opNuRho = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opNuRho->addTerm(new VecAtQP_ZOT(prob->getSolution()->getDOFVector(2), NULL));
  opNuRho->setUhOld(prob->getSolution()->getDOFVector(0));
  prob->addMatrixOperator(opNuRho, 2, 0, &minus3, &minus3); 
  prob->addVectorOperator(opNuRho, 2, &minus3, &minus3); 
  
  // 4*rho*grad(rho)*grad(rho)
  Operator *opRhoGradRho = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opRhoGradRho->addTerm(new VecGrad_FOT(prob->getSolution()->getDOFVector(0), 
					prob->getSolution()->getDOFVector(0), 
					new AMDiS::Mult2<WorldVector<double>, double, WorldVector<double> >));
  opRhoGradRho->setUhOld(prob->getSolution()->getDOFVector(0));
  prob->addMatrixOperator(opRhoGradRho, 2, 0, &eight, &eight);
  prob->addVectorOperator(opRhoGradRho, 2, &eight, &eight);
  Operator *opNormGradRho = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opNormGradRho->addTerm(new FctGradient_ZOT(prob->getSolution()->getDOFVector(0), new Norm2Sqr<double, WorldVector<double> >));
  prob->addMatrixOperator(opNormGradRho, 2, 0, &four, &four);
  
  // 2*grad(rho)*grad(nu)
  Operator *opGradRho = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opGradRho->addTerm(new VectorGradient_FOT(prob->getSolution()->getDOFVector(0), new AMDiS::Factor<WorldVector<double>, WorldVector<double> >(2.0)));
  prob->addMatrixOperator(opGradRho, 2, 2);
  Operator *opGradNu = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opGradNu->addTerm(new VectorGradient_FOT(prob->getSolution()->getDOFVector(2), new AMDiS::Factor<WorldVector<double>, WorldVector<double> >(2.0)));
  opGradNu->setUhOld(prob->getSolution()->getDOFVector(0));
  prob->addMatrixOperator(opGradNu, 2, 0);
  prob->addVectorOperator(opGradNu, 2);
  
  // -2*div(rho*grad(nu)), TODO: wie in Gleichung für nu den Term entwickeln
  prob->addMatrixOperator(opL, 2, 2, &two, &two);

  // mu
  prob->addMatrixOperator(opM, 2, 1);
};

} }
