/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#include "CahnHilliard_DD.h"
#include "Expressions.h"

namespace AMDiS { namespace base_problems {

void CahnHilliard_DD::fillOperators()
{
  const FiniteElemSpace* feSpace = prob->getFeSpace();  
  DOFVector<double>* c = prob->getSolution()->getDOFVector(1);
  
  
  // c
  Operator *opChMnew = new Operator(feSpace, feSpace);
  addZOT(opChMnew, valueOf(phase));
  
  Operator *opChMold = new Operator(feSpace, feSpace);
  addZOT(opChMold, valueOf(phase) * valueOf(c));
  
  // -nabla*(grad(c))
  Operator *opChL = new Operator(feSpace, feSpace);
  addSOT(opChL, valueOf(phase));
  
  
  // div(M(c)grad(mu)), with M(c)=gamma/4*(c^2-1)^2
  Operator *opChLM = new Operator(feSpace, feSpace);
  addSOT(opChLM, valueOf(phase) * gamma);
  
  // -2*c_old^3 + 3/2*c_old^2
  Operator *opChMPowExpl = new Operator(feSpace, feSpace);
  addZOT(opChMPowExpl, -2.0 * valueOf(phase) * pow<3>(valueOf(c)));
  if (doubleWell == 0)
    addZOT(opChMPowExpl, (3.0/2.0) * valueOf(phase) * pow<2>(valueOf(c)));

  // -3*c_old^2 * c
  Operator *opChMPowImpl = new Operator(feSpace, feSpace);
  addZOT(opChMPowImpl, -3.0 * valueOf(phase) * pow<2>(valueOf(c)));
  if (doubleWell == 0)
    addZOT(opChMPowImpl, 3.0 * valueOf(phase) * valueOf(c));
  else
    addZOT(opChMPowImpl, valueOf(phase));
  

  // mu + eps^2*laplace(c) + c - 3*(c_old^2)*c = -2*c_old^3 [+ BC]
  // ----------------------------------------------------------------------
  prob->addMatrixOperator(*opChMPowImpl,0,1);          		   // < c - 3*c*c_old^2 , psi >
  prob->addMatrixOperator(*opChL,0,1, &minusEpsSqr, &minusEpsSqr); // < -eps^2*grad(c) , grad(psi) >
  prob->addMatrixOperator(*opChMnew,0,0);              		   // < mu , psi >
  // . . . vectorOperators . . . . . . . . . . . . . . .
  prob->addVectorOperator(*opChMPowExpl,0);            		   // < -2*c_old^3 , psi >

  
  // dt(c) = laplace(mu) - u*grad(c)
  // -----------------------------------
  prob->addMatrixOperator(*opChMnew,1,1); 			   // < c , psi >
  prob->addMatrixOperator(*opChLM,1,0, getTau(), getTau());        // < tau*grad(mu) , grad(psi) >
  // . . . vectorOperators . . . . . . . . . . . . . . .
  prob->addVectorOperator(*opChMold,1);   			   // < c^old , psi >
  
}

} }
