#include "AMDiS.h"
#include "MyProblemInstat.h"
#include "Refinement.h"

using namespace AMDiS;

void MyProblemInstat::solveInitialProblem(AdaptInfo *adaptInfo)
{
  FUNCNAME("MyProblemInstat::solveInitialProblem()");


  string phiArh = "", rhoArh = ""; int nProc;
  Parameters::get("dendrites->read phi arh", phiArh);
  Parameters::get("dendrites->read rho arh", rhoArh);
  Parameters::get("dendrites->number of processors", nProc);
    
  if (phiArh != "" && rhoArh != "") {

    io::Arh2Reader::readFile(phiArh.c_str(), 
			problemStat->getSolution(0), NULL, NULL, true, nProc);
    io::Arh2Reader::readFile(rhoArh.c_str(), 
			problemStat->getSolution(1), NULL, NULL, true, nProc);
    
    oldSolution->getDOFVector(0)->copy(*(problemStat->getSolution()->getDOFVector(0)));
    oldSolution->getDOFVector(1)->copy(*(problemStat->getSolution()->getDOFVector(1)));

#ifdef HAVE_PARALLEL_DOMAIN_AMDIS    
    Parallel::MeshDistributor::globalMeshDistributor->checkMeshChange(false);
#endif
    return;
  }

  if (serial <= 0) {

    double r = 0.0;
    Parameters::get("dendrites->radius", r);

    AbstractFunction<double, WorldVector<double> > *signedDistFct = new Circle(r);
    AbstractFunction<double, WorldVector<double> > *initialPhaseField =
      new InitialPhaseField(signedDistFct);
    
    Refinement refinePhi(problemStat->getSolution()->getDOFVector(0),
			 initialPhaseField, 
			 problemStat->getRefinementManager(0),
			 problemStat->getCoarseningManager(0));
    refinePhi.refine(hInitInterface);

    if (problemStat->getMesh(0) != problemStat->getMesh(1)) {
      Refinement refineRho(problemStat->getSolution()->getDOFVector(1),
			   initialPhaseField, 
			   problemStat->getRefinementManager(1),
			   problemStat->getCoarseningManager(1));
      refineRho.refine(hInitInterface);
    }

#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
    Parallel::MeshDistributor::globalMeshDistributor->checkMeshChange(false);
#endif
    problemStat->getSolution()->getDOFVector(1)->set(-rhoInitial);

    delete signedDistFct;
    delete initialPhaseField;
  }
  
  problemStat->getMesh(0)->dofCompress();
  problemStat->getMesh(1)->dofCompress();
}


void MyProblemInstat::closeTimestep(AdaptInfo *adaptInfo)
{
  FUNCNAME("MyProblemInstat::closeTimestep()");

  ProblemInstat::closeTimestep(adaptInfo);
  
  if (refInterface) {
    Refinement refinePhi(problemStat->getSolution()->getDOFVector(0),
			 NULL,
			 problemStat->getRefinementManager(0),
			 problemStat->getCoarseningManager(0));
    refinePhi.refine(hInterface);
    
#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
    Parallel::MeshDistributor::globalMeshDistributor->checkMeshChange(false);
#endif
  }

  if (problemStat->getMesh(0)->getDim() == 3)
    return;

#ifndef HAVE_PARALLEL_DOMAIN_AMDIS

  /*
  mtl::dense_vector<double> localVec(problemStat->getFeSpace(0)->getBasisFcts()->getNumber());
  DOFVector<double> *phi = problemStat->getSolution()->getDOFVector(0);

  TraverseStack stack;
  ElInfo *elInfo = stack.traverseFirst(problemStat->getMesh(0), -1, 
				       Mesh::CALL_LEAF_EL | 
				       Mesh::FILL_COORDS);
  while (elInfo) {
    std::vector<int> zeroCoords(0);

    for (int i = 0; i < 3; i++)
      if (elInfo->getCoord(i)[1] == 0.0)
	zeroCoords.push_back(i);
    
    if (zeroCoords.size() == 2) {
      phi->getLocalVector(elInfo->getElement(), localVec);
    
      if (sign(localVec[zeroCoords[0]]) != sign(localVec[zeroCoords[1]])) {
	if (elInfo->getCoord(zeroCoords[0])[0] > elInfo->getCoord(zeroCoords[1])[0])
	  std::swap(zeroCoords[0], zeroCoords[1]);

	TEST_EXIT(localVec[zeroCoords[0]] > 0.0)("Error 1\n");
	TEST_EXIT(localVec[zeroCoords[1]] < 0.0)("Error 2\n");

	double m = (localVec[zeroCoords[0]] - localVec[zeroCoords[1]]) / 
	  (elInfo->getCoord(zeroCoords[1])[0] - elInfo->getCoord(zeroCoords[0])[0]);
	double xZero = 
	  elInfo->getCoord(zeroCoords[0])[0] + (localVec[zeroCoords[0]] / m);
	
	MSG("Tip-x position at t = %f : %f\n", adaptInfo->getTime(), xZero);	
      }
    }

    zeroCoords.clear();

    for (int i = 0; i < 3; i++)
      if (elInfo->getCoord(i)[0] == 0.0)
	zeroCoords.push_back(i);
    
    if (zeroCoords.size() == 2) {
      phi->getLocalVector(elInfo->getElement(), localVec);
    
      if (sign(localVec[zeroCoords[0]]) != sign(localVec[zeroCoords[1]])) {
	if (elInfo->getCoord(zeroCoords[0])[1] > elInfo->getCoord(zeroCoords[1])[1])
	  std::swap(zeroCoords[0], zeroCoords[1]);

	TEST_EXIT(localVec[zeroCoords[0]] > 0.0)("Error 1\n");
	TEST_EXIT(localVec[zeroCoords[1]] < 0.0)("Error 2\n");

	double m = (localVec[zeroCoords[0]] - localVec[zeroCoords[1]]) / 
	  (elInfo->getCoord(zeroCoords[1])[1] - elInfo->getCoord(zeroCoords[0])[1]);
	double yZero = 
	  elInfo->getCoord(zeroCoords[0])[1] + (localVec[zeroCoords[0]] / m);
	
	MSG("Tip-y position at t = %f : %f\n", adaptInfo->getTime(), yZero);	
      }
    }

    elInfo = stack.traverseNext(elInfo);
  }
  */

#endif

}
