/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef AMDIS_PETSC_PRECON_PFC_H
#define AMDIS_PETSC_PRECON_PFC_H

#include "parallel/PetscSolverGlobalBlockMatrix.h"

namespace AMDiS { namespace Parallel {

  struct PfcData {
    KSP kspM, kspMpK, kspMpK2;
    Mat matM, matK;
    double delta, tau;
  };

  class PetscSolverPfc : public PetscSolverGlobalBlockMatrix
  {
  public:
    typedef PetscSolverPfc                self;
    typedef PetscSolverGlobalBlockMatrix  super;
  
    /// Creator class
    struct Creator : public LinearSolverCreator
    {
      virtual ~Creator() {}
      LinearSolverInterface* create() { return new self(this->name); }
    };
    
    PetscSolverPfc(std::string name)
      : super(name),
	solverM(NULL),
	solverK(NULL),
	solverMpK(NULL),
	solverMpK2(NULL),
	useOldInitialGuess(false),
	tau(NULL),
	M0(1.0)
    { 	
      Parameters::get(initFileStr + "->use old initial guess", useOldInitialGuess);
    }

    void setData(double *tau_, double M0_ = 1.0)
    {
      tau = tau_;
      M0 = M0_;
    }

  protected:
    void initSolver(KSP &ksp);

    void initPreconditioner(PC pc);
    void exitPreconditioner(PC pc);
    
    PetscSolver* createSubSolver(int component, std::string kspPrefix);

  private:
    Mat matMpK;
    Mat matMpK2;
    PfcData data;

    PetscSolver *solverM, *solverK, *solverMpK, *solverMpK2;

    bool useOldInitialGuess;

    double *tau;
    double M0;
  };

} }

#endif // AMDIS_PETSC_PRECON_PFC_H
