/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef AMDIS_PETSC_PRECON_PFC_DIAG_H
#define AMDIS_PETSC_PRECON_PFC_DIAG_H

#include "parallel/PetscSolverGlobalBlockMatrix.h"

namespace AMDiS { namespace Parallel {

  struct PfcDiagData {
    KSP kspM, kspMpK, kspS;
    Mat matM, matK, matS;
    double delta, tau;
  };

  class PetscSolverPfcDiag : public PetscSolverGlobalBlockMatrix
  {
  public:
    typedef PetscSolverPfcDiag            self;
    typedef PetscSolverGlobalBlockMatrix  super;
  
    /// Creator class
    struct Creator : public LinearSolverCreator
    {
      virtual ~Creator() {}
      LinearSolverInterface* create() { return new self(this->name); }
    };
    
    PetscSolverPfcDiag(std::string name)
      : super(name),
	solverM(NULL),
	solverK(NULL),
	solverMpK(NULL),
	solverS(NULL),
	useOldInitialGuess(false),
	tau(NULL)
    { 	
      Parameters::get(initFileStr + "->use old initial guess", useOldInitialGuess);
    }

    void setData(double *tauPtr)
    {
      tau = tauPtr;
    }

  protected:
    void initSolver(KSP &ksp);

    void initPreconditioner(PC pc);
    void exitPreconditioner(PC pc);
    
    PetscSolver* createSubSolver(int component, std::string kspPrefix);

  private:
    Mat matMpK;
    Mat matS;
    PfcDiagData data;

    PetscSolver *solverM, *solverK, *solverMpK, *solverS;

    bool useOldInitialGuess;    
    SystemVector* solution;

    double *tau;
  };

} }

#endif // AMDIS_PETSC_PRECON_PFC_DIAG_H
